<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use \Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController;

class VP_Woo_Pont_ExpressOne {
	protected $api_url = 'https://webcas.expressone.hu/webservice/';
	protected $username = '';
	protected $password = '';
	protected $api_key = '';
	public $package_statuses = array();
	public $package_statuses_tracking = array();

	public function __construct() {
		$this->company_id = VP_Woo_Pont_Helpers::get_option('expressone_company_id');
		$this->username = VP_Woo_Pont_Helpers::get_option('expressone_username');
		$this->password = VP_Woo_Pont_Helpers::get_option('expressone_password');

		//Load settings
		add_filter('vp_woo_pont_global_settings', array($this, 'get_settings'));

		//Set supported statuses
		$this->package_statuses = array(
			'D00' => 'Sikeres kézbesítés',
			'OFD' => 'Kiadva futárnak',
			'DEP' => 'Depóba érkezett',
			'OUB' => 'Központi raktárat elhagyta',
			'INB' => 'Központi raktárba érkezett',
			'DLS' => 'Adat bekerült a rendszerbe',
			'E40' => 'Depó kiszállítási kapacitási hiánya miatt',
			'VV' => 'Vissza a feladónak',
			'E92' => 'Kézbesítési nap módosítása Rugalmas kézbesítés felületen',
			'V96' => 'Címzett visszautasította'
		);

		//Categorized for tracking page
		$this->package_statuses_tracking = array(
			'shipped' => array('DEP', 'OUB', 'INB', 'DLS', 'E92'),
			'delivery' => array('0FD'),
			'delivered' => array('D00'),
			'errors' => array('E40', 'VV', 'V96')
		);

		//Display a button in the orders admin page to manually run sync
		add_action( 'manage_posts_extra_tablenav', array( $this, 'display_sync_button' ), 9 );
		add_action( 'woocommerce_order_list_table_extra_tablenav', array( $this, 'display_sync_button' ), 9 );
		add_action( 'wp_ajax_vp_woo_pont_expressone_run_sync', array( $this, 'run_sync_with_ajax' ) );

	}

	public function get_settings($settings) {
		$expressone_settings = array(
			'section_expressone' => array(
				'title' => __( 'Express One settings', 'vp-woo-pont' ),
				'type' => 'vp_woo_pont_settings_title',
				'description' => __( 'Settings related to the Express One provider.', 'vp-woo-pont' ),
			),
			'expressone_company_id' => array(
				'title' => __('Company ID', 'vp-woo-pont'),
				'type' => 'text',
			),
			'expressone_username' => array(
				'title' => __('Username', 'vp-woo-pont'),
				'type' => 'text',
			),
			'expressone_password' => array(
				'title' => __('Password', 'vp-woo-pont'),
				'type' => 'password',
			),
			'expressone_customer_sms' => array(
				'title'    => __( 'Customer SMS Notification', 'vp-woo-pont' ),
				'type'     => 'checkbox',
				'description' => __('Send an SMS notification to the customer(extra service).', 'vp-woo-pont')
			),
			'expressone_groups' => array(
				'type' => 'vp_woo_pont_settings_checkboxes',
				'title' => __( 'Enabled point providers', 'vp-woo-pont' ),
				'options' => array(
					'omv' => __('OMV stations', 'vp-woo-pont'),
					'alzabox' => __('AlzaBox', 'vp-woo-pont'),
					'packeta' => __('Packeta', 'vp-woo-pont')
				),
				'default' => array('omv', 'alzabox', 'packeta')
			),
		);

		return $settings+$expressone_settings;
	}

	public function create_label($data) {

		//Get order details
		$order = wc_get_order($data['order_id']);

		//Set package weight
		$weight = ($data['package']['weight'] == 0) ? 1 : wc_get_weight($data['package']['weight'], 'kg');
		$weight = ceil($weight);

		//Setup basic data
		$item = array(
			'auth' => $this->get_auth(),
			'deliveries' => array(
				array(
					'post_date' => date_i18n('Y-m-d'),
					'consig' => array(
						'name' => $data['customer']['name'],
						'contact_name' => $data['customer']['name_with_company'],
						'city' => $order->get_shipping_city(),
						'street' => $order->get_shipping_address_1().' '.$order->get_shipping_address_2(),
						'country' => $order->get_shipping_country(),
						'post_code' => $order->get_shipping_postcode(),
						'phone' => $data['customer']['phone'],
					),
					'parcels' => array(
						'type' => 0,
						'qty' => 1,
						'weight' => $weight,
					),
					'services' => array(
						'delivery_type' => ($data['point_id']) ? 'D2S' : '24H',
						'insurance' => array(
							'enable' => true,
							'parcel_price' => (double) $order->get_total() //Net order total
						),
						'notification' => array(
							'email' => $data['customer']['email'],
							'sms' => (VP_Woo_Pont_Helpers::get_option('expressone_customer_sms', 'no') == 'yes') ? $data['customer']['phone'] : ''
						)
					),
					'ref_number' => $data['reference_number'],
					'note' => '',
					'invoice_number' => '',
				)
			),
			'labels' => array(
				'data_type' => 'PDF',
				'size' => 'A4',
				'pdf_etiket_position' => 0
			)
		);

		//If its a pont shipping method
		if($data['point_id']) {
			$item['deliveries']['services']['eone_shop'] = $data['point_id'];
		}

		//Check for COD
		if($data['package']['cod']) {
			$item['deliveries'][0]['services']['cod'] = array(
				'amount' => round($data['package']['total']/5, 0) * 5
			);
		}

		//So developers can modify
		$options = apply_filters('vp_woo_pont_expressone_label', $item, $data);

		//Logging
		VP_Woo_Pont()->log_debug_messages($options, 'expressone-create-label');

		//Try to create label
		$request = wp_remote_post( $this->api_url.'parcel/create_labels/response_format/json', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
			'timeout' => 60
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );

		//Check fo API errors
		if(!$response['successfull']) {
			VP_Woo_Pont()->log_error_messages($response, 'expressone-create-label');
			return new WP_Error( 'expressone_error_'.$response['error_code'], $response['error_messages'] );
		}

		//Check for other errors
		if(isset($response['response']['deliveries'][0]['message']) && $response['response']['deliveries'][0]['message'] != 'OK') {
			VP_Woo_Pont()->log_error_messages($response, 'expressone-create-label');
			return new WP_Error( 'expressone_error_unknown', __('Unknown error', 'vp-woo-pont') );
		}

		//Get parcel number
		$parcel = $response['response']['deliveries'][0]['data'];
		$parcel_number = $parcel['parcel_numbers'][0];

		//Try to save PDF file
		$pdf = base64_decode($response['response']['labels']['data']);
		$pdf_file = VP_Woo_Pont_Labels::get_pdf_file_path('expressone', $data['order_id']);
		VP_Woo_Pont_Labels::save_pdf_file($pdf, $pdf_file);

		//Create response
		$label = array();
		$label['id'] = $parcel_number;
		$label['number'] = $parcel_number;
		$label['pdf'] = $pdf_file['name'];

		//Return file name, package ID, tracking number which will be stored in order meta
		return $label;
	}

	public function void_label($data) {

		//Create request data
		$options = array(
			'auth' => $this->get_auth(),
			'parcel_number' => $data['parcel_number']
		);

		//Logging
		VP_Woo_Pont()->log_debug_messages($options, 'expressone-void-label');

		//Try to create label
		$request = wp_remote_post( $this->api_url.'parcel/delete_labels/response_format/json', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
			'timeout' => 60
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );

		//Check fo API errors
		if(!$response['successfull']) {
			//If INVALID_PARCEL_ID, that means it was deleted on webcas already, return true
			if($response['error_code'] == 20304) {
				$label = array();
				$label['success'] = true;
				return $label;
			} else {
				VP_Woo_Pont()->log_error_messages($response, 'expressone-create-label');
				return new WP_Error( 'expressone_error_'.$response['error_code'], $response['error_messages'] );
			}
		}

		//Check for success
		$label = array();
		$label['success'] = true;

		VP_Woo_Pont()->log_debug_messages($response, 'expressone-void-label-response');

		return $label;
	}

	//Return tracking link
	public function get_tracking_link($parcel_number, $order = false) {
		return 'https://tracking.expressone.hu/?plc_number='.esc_attr($parcel_number);
	}

	//Function to get tracking informations using API
	public function get_tracking_info($order) {

		//Parcel number
		$parcel_number = $order->get_meta('_vp_woo_pont_parcel_number');

		//Create request data
		$options = array(
			'auth' => $this->get_auth(),
			'parcel_number' => $parcel_number
		);

		//Try to create label
		$request = wp_remote_post( $this->api_url.'tracking/get_parcel_history/response_format/json', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
			'timeout' => 60
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );

		//Check fo API errors
		if(!$response['successfull']) {
			VP_Woo_Pont()->log_error_messages($response, 'expressone-create-label');
			return new WP_Error( 'expressone_error_'.$response['error_code'], $response['error_messages'] );
		}

		//Loop through and create standard events
		$tracking_info = array();
		foreach ($response['response']['history'] as $event) {
			$tracking_info[] = array(
				'date' => strtotime($event['created_at']),
				'event' => $event['event_code'],
				'label' => $event['event_name'],
				'comment' => $event['event_name'].' '.$event['event_location']
			);
		}

		return array_reverse($tracking_info);
	}

	public function get_auth() {
		return array(
			'company_id' => $this->company_id,
			'user_name' => $this->username,
			'password' => $this->password
		);
	}

	public function display_sync_button() {
		global $pagenow, $post_type;
		if ( class_exists( CustomOrdersTableController::class ) && function_exists( 'wc_get_container' ) && wc_get_container()->get( CustomOrdersTableController::class )->custom_orders_table_usage_is_enabled() ) {
			$pagenow = 'edit.php';
			$post_type = 'shop_order';
		}
		if( 'shop_order' === $post_type && 'edit.php' === $pagenow && VP_Woo_Pont_Helpers::is_provider_configured('expressone') && VP_Woo_Pont_Pro::is_pro_enabled()) {
			?>
			<div class="alignleft actions vp-woo-pont-expressone-start-sync">
				<a href="#" class="button tips" data-nonce="<?php echo wp_create_nonce( "vp_woo_pont_expressone_start_sync" ); ?>" data-tip="<?php esc_attr_e('Close pending Express One shipments','vp-woo-pont'); ?>">
					<i class="vp-woo-pont-provider-icon-expressone"></i>
					<span><?php esc_html_e('Daily closing', 'vp-woo-pont'); ?></span>
					<span class="dashicons dashicons-yes"></span>
				</a>
			</div>
			<?php
		}
	}

	public function run_sync_with_ajax() {
		check_ajax_referer( 'vp_woo_pont_expressone_start_sync', 'nonce' );
		if ( !current_user_can( 'edit_shop_orders' ) ) {
			wp_die( __( 'You do not have sufficient permissions to access this action.', 'vp-woo-pont' ) );
		}
		$send = $this->parcel_data_send();
		if(is_wp_error($send)) {
			wp_send_json_error();
		} else {
			wp_send_json_success($send);
		}
	}

	public function parcel_data_send() {

		//Create request data
		$options = array(
			'auth' => $this->get_auth(),
			'date' => date_i18n('Y-m-d'),
			'options' => array(
				'closing' => true
			)
		);

		//Try to create label
		$request = wp_remote_post( $this->api_url.'parcel_list/create_list/response_format/json', array(
			'body'    => json_encode($options),
			'headers' => array(
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
			),
			'timeout' => 60
		));

		//Check for errors
		if(is_wp_error($request)) {
			return $request;
		}

		//Parse response
		$response = wp_remote_retrieve_body( $request );
		$response = json_decode( $response, true );

		//Check fo API errors
		if(!$response['successfull']) {
			VP_Woo_Pont()->log_error_messages($response, 'expressone-create-label');
			return new WP_Error( 'expressone_error_'.$response['error_code'], $response['error_messages'] );
		}

		//Save PDF
		$pdf = base64_decode($response['pdf']);
		$pdf_file = VP_Woo_Pont_Labels::get_pdf_file_path('expressone', 'list-and-close');
		VP_Woo_Pont_Labels::save_pdf_file($pdf, $pdf_file);
		$pdf_file_url = $pdf_file['baseurl'].$pdf_file['name'];

		//Return download link
		return array('pdf' => $pdf_file_url);
	}

}
